/***************************************************************
 * PrEmptyFSDefn.h
 *
 * This files contains constants for empty force sensor.
 *
 ****************************************************************/

/*
 * modification history
 *----------------------
 *
 * 12/19/97: K.C. Chang: created.
 */

#ifndef _PrEmptyFSDefn_h
#define _PrEmptyFSDefn_h

#include "PrGlobalDefn.h"

#define PR_FORCESENSOR_ID   PR_EMPTY_ID
#define PR_FORCESENSOR_DOF  0

#endif // _PrEmptyFSDefn_h
