/***************************************************************
 * PrEmptyGripperDefn.h
 *
 * This files contains constants for empty gripper.
 *
 ****************************************************************/

/*
 * modification history
 *----------------------
 *
 * 12/19/97: K.C. Chang: created.
 */

#ifndef _PrEmptyGripperDefn_h
#define _PrEmptyGripperDefn_h

#include "PrGlobalDefn.h"

#define PR_HAND_ID                 PR_EMPTY_ID
#define PR_HAND_NOJ                0

#define PR_HAND_DIST_PER_COUNT     { 0.0 }
#define PR_HAND_GEARRATIO          { 1.0 }
#define PR_HAND_MOTORSCALE         { 0.0 }
#define PR_HAND_POSITION_AT_NEST   { 0.0 }
#define PR_HAND_ENCODER_COUNT_ZERO { 0 }

#endif // _PrEmptyGripperDefn_h
