/*
	File: GripperControl.h
	***************************************************************************

	Peter Thaulad
	03/12/2004

	Header file for Gripper control implementation using PIC18F microcontroller
	connected to STG board. 

*/

#define OPEN			1100
#define CLOSE			2000
#define STOP			100
#define PORTA_DIR			1
#define PGRIPPER_CHANNEL	6
#define PORTA				0
#define CONTROL_PORT		PORTA

class GripperControl
{
	private:
	ServoToGo *stg;

	public:
	GripperControl(ServoToGo *stg_card);
	~GripperControl();
	void gripperOpen(void);
	void gripperClose(void);
	void gripperStop(void);
	void gripperCommand(int goal);

};

