#ifndef _PrJr3ForceISADefn_h
#define _PrJr3ForceISADefn_h

#include "PrJr3Defn.h"

#define PR_FORCESENSOR_ID       PR_JR3_ISA_FS_ID
#define PR_FORCESENSOR_DOF      PR_JR3_CHANNEL_COUNT

#define PR_JR3_ISA_BASE_ADDR    0x0200  /* address of isa board #1     */
#define PR_JR3_ISA_ADDR_SPACE_SIZE  4   /* ISA board occupies 4 bytes  */

#endif
