#ifndef _PrJr3Defn_h
#define _PrJr3Defn_h

#include "PrGlobalDefn.h"

#define PR_JR3_CHANNEL_COUNT    6
#define PR_JR3_BIT_COUNT        16

#define PR_JR3_FULL_SCALE_ADDR  0x0080  /* full scale data    */
#define PR_JR3_FILTER0_ADDR     0x0090  /* filter 0 (unfiltered) data  */
#define PR_JR3_FILTER1_ADDR     0x0098  /* filter 1 (500 Hz) data      */
#define PR_JR3_FILTER2_ADDR     0x00A0  /* filter 2 (125 Hz) data      */
#define PR_JR3_FILTER3_ADDR     0x00A8  /* filter 3 (31.25 Hz) data    */
#define PR_JR3_FILTER4_ADDR     0x00B0  /* filter 4 (7.81 Hz) data     */
#define PR_JR3_FILTER5_ADDR     0x00B8  /* filter 5 (1.95 Hz) data     */
#define PR_JR3_FILTER6_ADDR     0x00C0  /* filter 6 (0.49 Hz) data     */
                          /* there is a time delay for each filter     */
                          /* Delay = 1 / Cutoff Frequency              */

#define PR_JR3_FADDR_FOR_RATE   0x00e2  /* force address to use for rate */
#define PR_JR3_RATE_ADDR        0x00c8  /* start of rate data            */
#define PR_JR3_RATE_DIVISOR     0x00e3  /* how often the rate is calculated */
#define PR_JR3_ERROR_COUNT_ADDR 0x00ee  /* See PrForceSensorJr3.h        */
#define PR_JR3_COUNTX_ADDR	0x00ef
#define PR_JR3_WARN_BITS_ADDR   0x00f0
#define PR_JR3_ERROR_BITS_ADDR  0x00f1

#define PR_JR3_FORCE_UNIT_ADDR  0x00fc  /* Address for specifying force unit */
#define PR_JR3_FORCE_UNIT       1       /* Enum type : Use Nm for force unit */
#define PR_JR3_COMM0_ADDR       0x00E7  /* command word 0 location       */
#define PR_JR3_ZERO_OFFSETS     0x0800
#define PR_JR3_SET_FULL_SCALE   0x0A00
#define PR_JR3_CPRIGHT_ADDR     0x0040  /* copyright location            */
#define PR_JR3_VER_ADDR         0x00f5  /* DSP version number location   */

#endif
