/***************************************************************
 * PrPC7166Defn.h
 *
 * This files contains constants for PC7166 board.
 *
 ****************************************************************/

/*
 * modification history
 *----------------------
 *
 * 10/26/97: K.C. Chang: created.
 */

#ifndef _PrPC7166Defn_h
#define _PrPC7166Defn_h

#define PC7166_BASE_ADDR              0x300
#define PC7166_ADDR_SPACE_SIZE        0xFF
#define PC7166_ENCODER_CHANNEL_COUNT  2
#define PC7166_ENCODER_BIT_COUNT      24

#define PC7166_ENCODER_MASTER_RESET    0x20
#define PC7166_ENCODER_INPUT_SETUP     0x68
#define PC7166_ENCODER_QUAD_X1         0xC1
#define PC7166_ENCODER_QUAD_X2         0xC2
#define PC7166_ENCODER_QUAD_X4         0xC3
#define PC7166_ENCODER_ADDR_RESET      0x01
#define PC7166_ENCODER_LATCH_CNTR      0x02
#define PC7166_ENCODER_CNTR_RESET      0x04
#define PC7166_ENCODER_PRESET_CTR      0x08

#endif // _PrPC7166Defn_h
