/***************************************************************
 * PrSTGDefn.h
 *
 * This files contains constants for the ServoToGo board.
 *
 ****************************************************************/

/*
 * modification history
 *----------------------
 *
 * 02/22/03: Weyn Ong: created.
 */

#ifndef _PrSTGDefn_h
#define _PrSTGDefn_h

#include "PrRobotDefn.h"

#define COUNTS_PER_VOLT (4096/10.0)     // DAC RESOLUTION
#define AMPS_PER_VOLT   2.000   // AMPLIFIER CURRENT GAIN
#define STG_ARM_IRQ 3
#define STG_ARM_ADDRESS             0x2A0
#define STG_ARM_ADDRESS_2_OFFSET    0x400
#define STG_ADDR_SPACE_SIZE_1       0x20
#define STG_ADDR_SPACE_SIZE_2       0x20

#endif // _PrSTGDefn_h
