/***************************************************************
 * PrOttoBockDefn.h
 *
 * This files contains constants for otto-bock gripper.
 *
 ****************************************************************/

/*
 * modification history
 *----------------------
 *
 * 12/19/97: K.C. Chang: created.
 */

#ifndef _PrOttoBockDefn_h
#define _PrOttoBockDefn_h

#include "PrGlobalDefn.h"

#define PR_HAND_ID                 PR_OTTO_BOCK_ID
#define PR_HAND_NOJ                1

#define PR_HAND_DIST_PER_COUNT     { 0.000293548 }     // 310/0.091m
#define PR_HAND_GEARRATIO          { 1.0 }
#define PR_HAND_MOTORSCALE         { 0.1 }
#define PR_HAND_POSITION_AT_NEST   { 0.0 }
#define PR_HAND_ENCODER_COUNT_ZERO { (0x80000) } 

#endif // _PrOttoBockDefn_h

