#ifndef _PrJr3ForcePCIDefn_h
#define _PrJr3ForcePCIDefn_h

#include "PrJr3Defn.h"

#define PR_FORCESENSOR_ID       PR_JR3_PCI_FS_ID
#define PR_FORCESENSOR_DOF      PR_JR3_CHANNEL_COUNT

#define PR_JR3_PCI_VENDOR_ID    0x1762
#define PR_JR3_PCI_DEVICE_ID    0x3112
#define PR_JR3_PCI_RESET_ADDR   0x18000
#define PR_JR3_PCI_DM_ADDR_MASK 0x6000  /* Not 0x10000 like the manual says! */
#define PR_JR3_PCI_NO_ADDR_MASK 0x40000 /* Tells the card to map to remaining
                                           8 bits of the 24-bit location. */

// Full scales to use.  Putting it in this PCI defn file because it doesn't
// appear to be needed in the ISA version.
#define PR_JR3_FULL_SCALE_FX    52      // Units in Nm
#define PR_JR3_FULL_SCALE_FY    51
#define PR_JR3_FULL_SCALE_FZ    141 
#define PR_JR3_FULL_SCALE_MX    36 
#define PR_JR3_FULL_SCALE_MY    35 
#define PR_JR3_FULL_SCALE_MZ    46 

/*
#define PR_JR3_FULL_SCALE_FX    10      // Units in Nm
#define PR_JR3_FULL_SCALE_FY    10 
#define PR_JR3_FULL_SCALE_FZ    10 
#define PR_JR3_FULL_SCALE_MX    10 
#define PR_JR3_FULL_SCALE_MY    10 
#define PR_JR3_FULL_SCALE_MZ    10 
*/
#endif
