#include "PrGlobalDefn.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <process.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/sched.h>

void main(int arc,char **argv)
{
  if (arc < 2)
    {
     printf("usage: manips [path]/servo\n");
     exit(1);
    }

  execl(argv[1],"servo",NULL);
  perror("execl:servo");
  exit(1);
}
