/***************************************************************
 * PrATIDefn.h
 *
 * This files contains constants for ATI board.
 *
 ****************************************************************/

/*
 * modification history
 *----------------------
 *
 * 12/12/97: K.C. Chang: created.
 */

#ifndef _PrATIDefn_h
#define _PrATIDefn_h

#include "PrGlobalDefn.h"

#define PR_FORCESENSOR_ID     PR_ATI_FS_ID

#define PR_ATI_CHANNEL_COUNT  6
#define PR_FORCESENSOR_DOF    PR_ATI_CHANNEL_COUNT

#define PR_ATI_BIT_COUNT      16

#define PR_ATI_BASE_ADDR      0x0280 // base I/O address of FT Card
#define PR_ATI_ADDR_SPACE_SIZE 0x0400   // XXX Guessed value
#define PR_ATI_ADDR_REG       PR_ATI_BASE_ADDR
#define PR_ATI_DATA_REG       0x0282
#define PR_ATI_STATUS_REG     0x0284
#define PR_ATI_IRQ            7     	// interrupt number
#define PR_ATI_INTERRUPT_MODE 0 // polling vs interrupt mode
#define PR_ATI_CLEAR          0x0000

#define PR_ATI_ACK_BIT        0x0010
#define PR_ATI_INTERRUPT_BIT  0x0080

#define PR_ATI_READ_COMMAND   0x0100
#define PR_ATI_READ_MODE      0x0000

#define PR_ATI_COMMAND_BLOCK  0x0000
#define PR_ATI_DATA_BLOCK     0x0100

#define PR_ATI_ACK_ADDR       0x03FE
#define PR_ATI_INTERRUPT_ADDR 0x03FF
#define PR_ATI_MONITOR_ADDR   0x0101
#define PR_ATI_FORCE_ADDR     0x0102

#endif // _PrATIDefn_h
