/***************************************************************
 * PrEmptyBaseDefn.h
 *
 * This files contains constants for empty base.
 *
 ****************************************************************/

/*
 * modification history
 *----------------------
 *
 * 12/19/97: K.C. Chang: created.
 */

#ifndef _PrEmptyBaseDefn_h
#define _PrEmptyBaseDefn_h

#include "PrGlobalDefn.h"

#define PR_BODY_ID                    PR_EMPTY_ID
#define PR_BODY_NOJ                   0

#define PR_BODY_DIST_PER_COUNT        { 0.0 }
#define PR_BODY_GEARRATIO             { 1.0 }
#define PR_BODY_MOTORSCALE            { 0.0 }
#define PR_BODY_POSITION_AT_NEST      { 0.0 }
#define PR_BODY_ENCODER_COUNT_ZERO    { 0 }

#endif // _PrEmptyBaseDefn_h

