/*
 * File:   puma.h
 *
 * This file provides constants to 
 * gather.cpp, user.cpp, and libpuma.cpp.
 */

/*
 * modification history
 *----------------------
 *
 * 03/28/98: K.C. Chang: created.
 */

#ifndef _puma_h
#define _puma_h

#define PUMA_NOJ                     6

#define PUMA_FS_OFFSET              0.091
#define PUMA_EE_OFFSET              0.192

#define PUMA_OK                     1
#define PUMA_ERROR                  0

#define PUMA_HAND_OPEN              0
#define PUMA_HAND_CLOSE             1

// Change the following #define to select which board/configuration
// to compile this library for.
#define BOARD_TO_USE PrIPBoardSTG
// #define BOARD_TO_USE PrIPBoardTRC

#endif /* _puma_h */

