/***************************************************************
 * PrTrackDefn.h
 *
 * This files contains constants for track gripper.
 *
 ****************************************************************/

/*
 * modification history
 *----------------------
 *
 * 12/19/97: K.C. Chang: created.
 */

#ifndef _PrTrackDefn_h
#define _PrTrackDefn_h

#include "PrGlobalDefn.h"

#define PR_BODY_ID                    PR_TRACK_ID
#define PR_BODY_NOJ                   1

//Pittman's DC motorGM14604B910
#define PR_BODY_TORQUE_CONSTANT       { 0.0306 } 
#define PR_BODY_AMP_CONSTANT          { 1.0 }
// torque const = 0.0306 N-m/A
// radius = 1.75/2 in
// amp const = 1.0 A/V
// = (1.0/(24.0*0.0306*1.0/(1.75*0.0254/2.0))) // V/N

#define PR_BODY_DIST_PER_COUNT        { 0.00000637271 }   // 166648/1.062m
#define PR_BODY_GEARRATIO             { 24.0 }
#define PR_BODY_MOTORSCALE            { 0.03026 }
#define PR_BODY_POSITION_AT_NEST      { 0.0 }
#define PR_BODY_ENCODER_COUNT_ZERO    { (0x800000) } 

#endif // _PrTrackDefn_h
