/***************************************************************
 * PrForceSensorJr3ISA.h
 *
 * Concrete derived class of PrForceSensorJr3 for ISA board.
 *
 ****************************************************************/

/*
 * modification history
 *----------------------
 *
 * 02/24/03: Weyn Ong : Created; based on code by Jaeheung Park.
 */

#ifndef _PrForceSensorJr3ISA_h
#define _PrForceSensorJr3ISA_h

#include "PrForceSensorJr3.h"

class PrForceSensorJr3ISA : public PrForceSensorJr3 {
  public:
    // Most of the public functions are in the base class PrforceSensorJr3.

    PrForceSensorJr3ISA();

  protected:
    unsigned short readJr3( int addr );
    void writeJr3( int addr, int data );

  private:
    void delay();
};

#endif // _PrForceSensorJr3ISA_h
