/*
 * File:   cs225.h
 *
 * This file provides constants to 
 * gather.cpp, user.cpp, and libcs225.cpp.
 */

/*
 * modification history
 *----------------------
 *
 * 03/28/98: K.C. Chang: created.
 */

#ifndef _cs225_h
#define _cs225_h

#define CS225_BUFFER_SIZE            256
#define CS225_GATHER_DATA_NUM        7
#define CS225_PUMA_NOJ               6
#define CS225_ANGLE_OFFSET           ((float)0.0)//((float)0.046837113)

#define CS225_USER_SERVO_TCP_PORT    5100
#define CS225_USER_GATHER_TCP_PORT   5200
#define CS225_GATHER_SERVO_TCP_PORT  5300

#define CS225_TYPE_CHAR              0
#define CS225_TYPE_ACTION            1
#define CS225_TYPE_GATHER_DATA       2

#define CS225_ACTION_EXIT            0
#define CS225_ACTION_GATHER_START    1
#define CS225_ACTION_GATHER_END      2

typedef struct
{
  float data[CS225_GATHER_DATA_NUM];
} GatherMesg;

/*
 * Function: HandleGatherMessage
 * Usage:    HandleGatherMessage()
 *
 * handles messages from and to the gather process.
 * called in WaitNextTick()
 */
void HandleGatherMessage();

#endif /* cs225.h */

